@@ -7,6 +7,7 @@
 
 
 import re
+import types
 
 from . import attributes  # noqa
 from .base import _class_to_mapper  # noqa
@@ -363,7 +364,6 @@ class ORMAdapter(sql_util.ColumnAdapter):
         entity,
         equivalents=None,
         adapt_required=False,
-        chain_to=None,
         allow_label_resolve=True,
         anonymize_labels=False,
     ):
@@ -381,7 +381,6 @@ class ORMAdapter(sql_util.ColumnAdapter):
             self,
             selectable,
             equivalents,
-            chain_to,
             adapt_required=adapt_required,
             allow_label_resolve=allow_label_resolve,
             anonymize_labels=anonymize_labels,
@@ -396,21 +395,30 @@ class ORMAdapter(sql_util.ColumnAdapter):
 class AliasedClass(object):
     r"""Represents an "aliased" form of a mapped class for usage with Query.
 
-    The ORM equivalent of a :func:`sqlalchemy.sql.expression.alias`
+    The ORM equivalent of a :func:`~sqlalchemy.sql.expression.alias`
     construct, this object mimics the mapped class using a
-    __getattr__ scheme and maintains a reference to a
+    ``__getattr__`` scheme and maintains a reference to a
     real :class:`~sqlalchemy.sql.expression.Alias` object.
 
-    Usage is via the :func:`.orm.aliased` function, or alternatively
-    via the :func:`.orm.with_polymorphic` function.
-
-    Usage example::
+    A primary purpose of :class:`.AliasedClass` is to serve as an alternate
+    within a SQL statement generated by the ORM, such that an existing
+    mapped entity can be used in multiple contexts.   A simple example::
 
         # find all pairs of users with the same name
         user_alias = aliased(User)
         session.query(User, user_alias).\
                         join((user_alias, User.id > user_alias.id)).\
-                        filter(User.name==user_alias.name)
+                        filter(User.name == user_alias.name)
+
+    :class:`.AliasedClass` is also capable of mapping an existing mapped
+    class to an entirely new selectable, provided this selectable is column-
+    compatible with the existing mapped selectable, and it can also be
+    configured in a mapping as the target of a :func:`.relationship`.
+    See the links below for examples.
+
+    The :class:`.AliasedClass` object is constructed typically using the
+    :func:`.orm.aliased` function.   It also is produced with additional
+    configuration when using the :func:`.orm.with_polymorphic` function.
 
     The resulting object is an instance of :class:`.AliasedClass`.
     This object implements an attribute scheme which produces the
@@ -429,8 +437,17 @@ class AliasedClass(object):
 
     The resulting inspection object is an instance of :class:`.AliasedInsp`.
 
-    See :func:`.aliased` and :func:`.with_polymorphic` for construction
-    argument descriptions.
+
+    .. seealso::
+
+        :func:`.aliased`
+
+        :func:`.with_polymorphic`
+
+        :ref:`relationship_aliased_class`
+
+        :ref:`relationship_to_window_function`
+
 
     """
 
@@ -479,34 +496,28 @@ class AliasedClass(object):
         except KeyError:
             raise AttributeError()
         else:
-            for base in _aliased_insp._target.__mro__:
-                try:
-                    attr = object.__getattribute__(base, key)
-                except AttributeError:
-                    continue
-                else:
-                    break
-            else:
-                raise AttributeError(key)
+            target = _aliased_insp._target
+            # maintain all getattr mechanics
+            attr = getattr(target, key)
 
-        if isinstance(attr, PropComparator):
-            ret = attr.adapt_to_entity(_aliased_insp)
-            setattr(self, key, ret)
-            return ret
-        elif hasattr(attr, "func_code"):
-            is_method = getattr(_aliased_insp._target, key, None)
-            if is_method and is_method.__self__ is not None:
-                return util.types.MethodType(attr.__func__, self, self)
-            else:
-                return None
-        elif hasattr(attr, "__get__"):
-            ret = attr.__get__(None, self)
-            if isinstance(ret, PropComparator):
-                return ret.adapt_to_entity(_aliased_insp)
-            else:
-                return ret
-        else:
-            return attr
+        # attribute is a method, that will be invoked against a
+        # "self"; so just return a new method with the same function and
+        # new self
+        if hasattr(attr, "__call__") and hasattr(attr, "__self__"):
+            return types.MethodType(attr.__func__, self)
+
+        # attribute is a descriptor, that will be invoked against a
+        # "self"; so invoke the descriptor against this self
+        if hasattr(attr, "__get__"):
+            attr = attr.__get__(None, self)
+
+        # attributes within the QueryableAttribute system will want this
+        # to be invoked so the object can be adapted
+        if hasattr(attr, "adapt_to_entity"):
+            attr = attr.adapt_to_entity(_aliased_insp)
+            setattr(self, key, attr)
+
+        return attr
 
     def __repr__(self):
         return "<AliasedClass at 0x%x; %s>" % (
@@ -514,6 +525,9 @@ class AliasedClass(object):
             self._aliased_insp._target.__name__,
         )
 
+    def __str__(self):
+        return str(self._aliased_insp)
+
 
 class AliasedInsp(InspectionAttr):
     """Provide an inspection interface for an
@@ -566,14 +580,36 @@ class AliasedInsp(InspectionAttr):
     ):
         self.entity = entity
         self.mapper = mapper
-        self.selectable = selectable
+        self.selectable = (
+            self.persist_selectable
+        ) = self.local_table = selectable
         self.name = name
-        self.with_polymorphic_mappers = with_polymorphic_mappers
         self.polymorphic_on = polymorphic_on
         self._base_alias = _base_alias or self
         self._use_mapper_path = _use_mapper_path
         self.represents_outer_join = represents_outer_join
 
+        if with_polymorphic_mappers:
+            self._is_with_polymorphic = True
+            self.with_polymorphic_mappers = with_polymorphic_mappers
+            self._with_polymorphic_entities = []
+            for poly in self.with_polymorphic_mappers:
+                if poly is not mapper:
+                    ent = AliasedClass(
+                        poly.class_,
+                        selectable,
+                        base_alias=self,
+                        adapt_on_names=adapt_on_names,
+                        use_mapper_path=_use_mapper_path,
+                    )
+
+                    setattr(self.entity, poly.class_.__name__, ent)
+                    self._with_polymorphic_entities.append(ent._aliased_insp)
+
+        else:
+            self._is_with_polymorphic = False
+            self.with_polymorphic_mappers = [mapper]
+
         self._adapter = sql_util.ColumnAdapter(
             selectable,
             equivalents=mapper._equivalent_columns,
@@ -584,20 +620,6 @@ class AliasedInsp(InspectionAttr):
         self._adapt_on_names = adapt_on_names
         self._target = mapper.class_
 
-        for poly in self.with_polymorphic_mappers:
-            if poly is not mapper:
-                setattr(
-                    self.entity,
-                    poly.class_.__name__,
-                    AliasedClass(
-                        poly.class_,
-                        selectable,
-                        base_alias=self,
-                        adapt_on_names=adapt_on_names,
-                        use_mapper_path=_use_mapper_path,
-                    ),
-                )
-
     is_aliased_class = True
     "always returns True"
 
@@ -661,6 +683,17 @@ class AliasedInsp(InspectionAttr):
         else:
             assert False, "mapper %s doesn't correspond to %s" % (mapper, self)
 
+    @util.memoized_property
+    def _get_clause(self):
+        onclause, replacemap = self.mapper._get_clause
+        return (
+            self._adapter.traverse(onclause),
+            {
+                self._adapter.traverse(col): param
+                for col, param in replacemap.items()
+            },
+        )
+
     @util.memoized_property
     def _memoized_values(self):
         return {}
@@ -685,6 +718,19 @@ class AliasedInsp(InspectionAttr):
             with_poly,
         )
 
+    def __str__(self):
+        if self._is_with_polymorphic:
+            return "with_polymorphic(%s, [%s])" % (
+                self._target.__name__,
+                ", ".join(
+                    mp.class_.__name__
+                    for mp in self.with_polymorphic_mappers
+                    if mp is not self.mapper
+                ),
+            )
+        else:
+            return "aliased(%s)" % (self._target.__name__,)
+
 
 inspection._inspects(AliasedClass)(lambda target: target._aliased_insp)
 inspection._inspects(AliasedInsp)(lambda target: target)
@@ -769,9 +815,6 @@ def aliased(element, alias=None, name=None, flat=False, adapt_on_names=False):
      wouldn't have any "column correspondence" to the actual
      ``UnitPrice.price`` column as it is not a proxy of the original.
 
-     .. versionadded:: 0.7.3
-
-
     """
     if isinstance(element, expression.FromClause):
         if adapt_on_names:
@@ -975,7 +1018,7 @@ class _ORMJoin(expression.Join):
                 dest_selectable=adapt_to,
                 source_polymorphic=True,
                 dest_polymorphic=True,
-                of_type=right_info.mapper,
+                of_type_mapper=right_info.mapper,
             )
 
             if sj is not None:
@@ -1085,8 +1128,10 @@ def join(
     See :meth:`.Query.join` for information on modern usage
     of ORM level joins.
 
-    .. versionchanged:: 0.8.1 - the ``join_to_left`` parameter
-       is no longer used, and is deprecated.
+    .. deprecated:: 0.8
+
+        the ``join_to_left`` parameter is deprecated, and will be removed
+        in a future release.  The parameter has no effect.
 
     """
     return _ORMJoin(left, right, onclause, isouter, full)
@@ -1161,8 +1206,6 @@ def was_deleted(object_):
     This is regardless of whether or not the object is
     persistent or detached.
 
-    .. versionadded:: 0.8.0
-
     .. seealso::
 
         :attr:`.InstanceState.was_deleted`
@@ -1193,6 +1236,41 @@ def _entity_corresponds_to(given, entity):
     return entity.common_parent(given)
 
 
+def _entity_corresponds_to_use_path_impl(given, entity):
+    """determine if 'given' corresponds to 'entity', in terms
+    of a path of loader options where a mapped attribute is taken to
+    be a member of a parent entity.
+
+    e.g.::
+
+        someoption(A).someoption(A.b)  # -> fn(A, A) -> True
+        someoption(A).someoption(C.d)  # -> fn(A, C) -> False
+
+        a1 = aliased(A)
+        someoption(a1).someoption(A.b) # -> fn(a1, A) -> False
+        someoption(a1).someoption(a1.b) # -> fn(a1, a1) -> True
+
+        wp = with_polymorphic(A, [A1, A2])
+        someoption(wp).someoption(A1.foo)  # -> fn(wp, A1) -> False
+        someoption(wp).someoption(wp.A1.foo)  # -> fn(wp, wp.A1) -> True
+
+
+    """
+    if given.is_aliased_class:
+        return (
+            entity.is_aliased_class
+            and not entity._use_mapper_path
+            and (given is entity or given in entity._with_polymorphic_entities)
+        )
+    elif not entity.is_aliased_class:
+        return given.common_parent(entity.mapper)
+    else:
+        return (
+            entity._use_mapper_path
+            and given in entity.with_polymorphic_mappers
+        )
+
+
 def _entity_isa(given, mapper):
     """determine if 'given' "is a" mapper, in terms of the given
     would load rows of type 'mapper'.
@@ -1234,9 +1312,6 @@ def randomize_unitofwork():
     This utility is also available when running the test suite via the
     ``--reversetop`` flag.
 
-    .. versionadded:: 0.8.1 created a standalone version of the
-       ``--reversetop`` feature.
-
     """
     from sqlalchemy.orm import unitofwork, session, mapper, dependency
     from sqlalchemy.util import topological
