@@ -1,15 +1,20 @@
 from .. import config
 from .. import fixtures
 from ..assertions import eq_
+from ..assertions import in_
 from ..schema import Column
 from ..schema import Table
 from ... import bindparam
+from ... import case
+from ... import false
 from ... import func
 from ... import Integer
 from ... import literal_column
+from ... import null
 from ... import select
 from ... import String
 from ... import testing
+from ... import true
 from ... import tuple_
 from ... import union
 from ... import util
@@ -366,6 +371,7 @@ class ExpandingBoundInTest(fixtures.TablesTest):
             Column("id", Integer, primary_key=True),
             Column("x", Integer),
             Column("y", Integer),
+            Column("z", String(50)),
         )
 
     @classmethod
@@ -373,16 +379,62 @@ class ExpandingBoundInTest(fixtures.TablesTest):
         config.db.execute(
             cls.tables.some_table.insert(),
             [
-                {"id": 1, "x": 1, "y": 2},
-                {"id": 2, "x": 2, "y": 3},
-                {"id": 3, "x": 3, "y": 4},
-                {"id": 4, "x": 4, "y": 5},
+                {"id": 1, "x": 1, "y": 2, "z": "z1"},
+                {"id": 2, "x": 2, "y": 3, "z": "z2"},
+                {"id": 3, "x": 3, "y": 4, "z": "z3"},
+                {"id": 4, "x": 4, "y": 5, "z": "z4"},
             ],
         )
 
     def _assert_result(self, select, result, params=()):
         eq_(config.db.execute(select, params).fetchall(), result)
 
+    def test_multiple_empty_sets(self):
+        # test that any anonymous aliasing used by the dialect
+        # is fine with duplicates
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(table.c.x.in_(bindparam("q", expanding=True)))
+            .where(table.c.y.in_(bindparam("p", expanding=True)))
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(stmt, [], params={"q": [], "p": []})
+
+    @testing.requires.tuple_in
+    def test_empty_heterogeneous_tuples(self):
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(
+                tuple_(table.c.x, table.c.z).in_(
+                    bindparam("q", expanding=True)
+                )
+            )
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(stmt, [], params={"q": []})
+
+    @testing.requires.tuple_in
+    def test_empty_homogeneous_tuples(self):
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(
+                tuple_(table.c.x, table.c.y).in_(
+                    bindparam("q", expanding=True)
+                )
+            )
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(stmt, [], params={"q": []})
+
     def test_bound_in_scalar(self):
         table = self.tables.some_table
 
@@ -412,6 +464,88 @@ class ExpandingBoundInTest(fixtures.TablesTest):
             stmt, [(2,), (3,), (4,)], params={"q": [(2, 3), (3, 4), (4, 5)]}
         )
 
+    @testing.requires.tuple_in
+    def test_bound_in_heterogeneous_two_tuple(self):
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(
+                tuple_(table.c.x, table.c.z).in_(
+                    bindparam("q", expanding=True)
+                )
+            )
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(
+            stmt,
+            [(2,), (3,), (4,)],
+            params={"q": [(2, "z2"), (3, "z3"), (4, "z4")]},
+        )
+
+    def test_empty_set_against_integer(self):
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(table.c.x.in_(bindparam("q", expanding=True)))
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(stmt, [], params={"q": []})
+
+    def test_empty_set_against_integer_negation(self):
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(table.c.x.notin_(bindparam("q", expanding=True)))
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(stmt, [(1,), (2,), (3,), (4,)], params={"q": []})
+
+    def test_empty_set_against_string(self):
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(table.c.z.in_(bindparam("q", expanding=True)))
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(stmt, [], params={"q": []})
+
+    def test_empty_set_against_string_negation(self):
+        table = self.tables.some_table
+
+        stmt = (
+            select([table.c.id])
+            .where(table.c.z.notin_(bindparam("q", expanding=True)))
+            .order_by(table.c.id)
+        )
+
+        self._assert_result(stmt, [(1,), (2,), (3,), (4,)], params={"q": []})
+
+    def test_null_in_empty_set_is_false(self):
+        stmt = select(
+            [
+                case(
+                    [
+                        (
+                            null().in_(
+                                bindparam("foo", value=(), expanding=True)
+                            ),
+                            true(),
+                        )
+                    ],
+                    else_=false(),
+                )
+            ]
+        )
+        in_(config.db.execute(stmt).fetchone()[0], (False, 0))
+
 
 class LikeFunctionsTest(fixtures.TablesTest):
     __backend__ = True
